@@ -47,11 +47,15 @@ class Dimensionality(dict):
     @property
     def unicode(self):
         return markup.format_units_unicode(self)
-        
+
     @property
     def latex(self):
         return markup.format_units_latex(self)
 
+    @property
+    def html(self):
+        return markup.format_units_html(self)
+
     def __hash__(self):
         res = hash(unit_registry['dimensionless'])
         for key in sorted(self.keys(), key=operator.attrgetter('format_order')):
@@ -285,6 +289,12 @@ p_dict[np.remainder] = _d_check_uniform
 p_dict[np.floor_divide] = _d_check_uniform
 p_dict[np.arctan2] = _d_check_uniform
 p_dict[np.hypot] = _d_check_uniform
+p_dict[np.equal] = _d_check_uniform
+p_dict[np.not_equal] = _d_check_uniform
+p_dict[np.less] = _d_check_uniform
+p_dict[np.less_equal] = _d_check_uniform
+p_dict[np.greater] = _d_check_uniform
+p_dict[np.greater_equal] = _d_check_uniform
 
 def _d_power(q1, q2, out=None):
     if getattr(q2, 'dimensionality', None):
@@ -309,6 +319,7 @@ p_dict[np.reciprocal] = _d_reciprocal
 
 def _d_copy(q1, out=None):
     return q1.dimensionality
+p_dict[np.fabs] = _d_copy
 p_dict[np.absolute] = _d_copy
 p_dict[np.conjugate] = _d_copy
 p_dict[np.negative] = _d_copy
