@@ -1,8 +1,5 @@
 import sys
-if sys.version.startswith('2.6') or sys.version.startswith('3.1'):
-    import unittest2 as unittest
-else:
-    import unittest
+import unittest
 
 import numpy as np
 
